@@ -9,7 +9,7 @@ from yt_dlp import  YoutubeDL
 from rich.console import Console
 from rich import print
 from bilibili_api import search as bilibili_search
-
+import csv
 import difflib
 
 # 支持的模型列表
@@ -34,6 +34,9 @@ def  extract_info(url):
             })
     
     info = ydl.extract_info(url, download=False)
+    if info == None:
+        console.log(f"{url}解析失败 请检查网址是否正确!")
+        return None
     return info
 
 def download(url:str,title:str=None,cover_url:str=None):
@@ -43,7 +46,23 @@ def download(url:str,title:str=None,cover_url:str=None):
         name (str): 歌曲[-歌手]
         cover_url (str, optional): 封面url. Defaults to None.
     """ 
-    with console.status("[bold green]正在下载...\n") as status:
+    info = extract_info(url)
+    if info == None:
+        return
+    try:
+        url = info['url']
+    except:
+        try:
+            url = info['webpage_url']
+        except:
+            print('获取下载地址失败!')
+            return
+    try:
+        download_title = info['title']
+    except:
+        print('获取标题失败!')
+        return
+    with console.status(f"[bold green]正在下载{download_title}...\n") as status:
         # 多平台强制删除temp文件夹
         try:
             # 判断文件夹是否存在
@@ -62,9 +81,9 @@ def download(url:str,title:str=None,cover_url:str=None):
             if title.find('(') != -1 and title.endswith(')') and len(title.split('(')[1].split(')')[0])!=0:
                 # 去除专辑格式
                 album = title.split('(')[1].split(')')[0]
-                title = title.rsplit('(',1)[0].strip().replace('/','').replace('\\','').replace('|','').replace('?','').replace('*','')
+                title = title.rsplit('(',1)[0].strip().replace('/','').replace('\\','').replace('⧸',' ').replace('⧹',' ').replace('|',' ').replace('?',' ').replace('*',' ')
             else:
-                title = title.strip().replace('/','').replace('\\','').replace('|','').replace('?','').replace('*','')
+                title = title.strip().replace('/','').replace('\\','').replace('⧸',' ').replace('⧹',' ').replace('|',' ').replace('?',' ').replace('*',' ')
             outtmpl = f'{title}.%(ext)s'
         else:
             outtmpl = '%(title)s.%(ext)s'
@@ -84,7 +103,6 @@ def download(url:str,title:str=None,cover_url:str=None):
             }],
         }
         with YoutubeDL(ydl_opts) as ydl:
-            info = extract_info(url)
             ydl.download([url])
             
         # 解决不规则标题引起的控制台乱码问题
@@ -155,8 +173,59 @@ def download(url:str,title:str=None,cover_url:str=None):
         else:
             os.system('rm -rf temp')
         console.log(f"下载完成!")
+        return info
     
-    
+
+def batch_download(csv_path:str):
+    """批量下载
+
+    Args:
+        csv_path (str): _description_
+    """    
+    with open(csv_path,'r',encoding='utf-8') as f:
+        reader = csv.reader(f)
+        # 去除第一行
+        skiped = True
+        for row in reader:
+            if skiped:
+                skiped = False
+                continue
+            try:
+                url = row[0]
+            except:
+                continue
+            try:
+                title = row[1] if row[1].strip()!='' else None
+            except:
+                title = None
+            try:
+                cover_url = row[2] if row[2].strip()!='' else None
+            except:
+                cover_url = None
+            try:
+                start_time = row[3] if row[3].strip()!='' else None
+            except:
+                start_time = None
+            try:
+                end_time = row[4] if row[4].strip()!='' else None
+            except:
+                end_time = None
+            try:
+                instrumental = row[5] if row[5].strip()!='' else None
+            except:
+                instrumental = 'false'
+            # 下载
+            info = download(url,title,cover_url)
+            if title!=None:
+                title = title.strip().replace('/','').replace('\\','').replace('⧸',' ').replace('⧹',' ').replace('|',' ').replace('?',' ').replace('*',' ')
+            # 剪辑
+            title = info['title']
+            if start_time!=None and end_time!=None:
+                clip(f'{title}.mp3',start_time,end_time)
+            if instrumental == 'true' or instrumental == 'True':
+                # 提取伴奏
+                extract_accompaniment(f'{title}.mp3')
+
 def clip(path:str,start:str,end:str):
     """剪辑音乐
 
@@ -282,7 +351,7 @@ async def search(name:str):
         name (str): 歌曲名称
     """ 
     with console.status("[bold green]搜索中...") as status:
-        
+        name = name.replace('⧸','/').replace('⧹','/').replace('⧺','+').replace('⧻','+').replace('⧼','<').replace('⧽','>').replace('⧾','>').replace('⧿','>')
         tasks = [
                 asyncio.create_task(search_youtube(name)), 
                 asyncio.create_task(search_bilibili(name))]
@@ -372,6 +441,8 @@ def main(args=None):
             '搜索: mk -s name\n'
             '剪辑: mk -c mp3path start end\n'
             '提取伴奏: mk -e mp3path \[model_name]\n'
+            '生成批量模板: mk -t\n'
+            '批量下载: mk csv_path\n'
             '---------------------------------------------\n' 
             )
         return
@@ -425,29 +496,101 @@ def main(args=None):
         except:
             model_name = None
         extract_accompaniment(path,model_name)
+    elif flag == '-t':
+        # 生成批量模板
+        with open('template.csv','w',encoding='utf-8',newline='') as f:
+            writer = csv.writer(f)
+            # 写入表头  
+            
+            # 下载地址 标题 封面url 开始时间 结束时间 是否生成伴奏(true or false)
+            writer.writerow(['url','title','cover_url','start_time','end_time','instrumental'])
+        print('生成成功!')
     else:
-        # 默认下载
         # 判断flag是否是网址
         if flag.startswith(('http://','https://')):
             url = flag
-            if len(args) == 1:
-                download(url,None,None)
-            elif len(args) == 2:
-                title_or_url = args[1]
-                if title_or_url.startswith(('http://','https://')):
-                    download(url,None,title_or_url)
-                else:
-                    download(url,title_or_url,None)
-            elif (len(args) == 3):
-                if args[1].startswith(('http://','https://')):
-                    print('歌曲名称不合法!')
+            # 如果url后面跟着|,  且本身就是一个播放列表 则批量下载 根据|后面的列表序号来筛选 (只对youtube音源有效) 序号之间用,分隔
+            if url.__contains__('youtube.com') and url.find('|') != -1 and url.find('list=') != -1:
+                raw_url = url.split('|')[0]
+                info = extract_info(raw_url)
+                try:
+                    list_url  = info['url'] if info['url']!=None else info['webpage_url']
+                except:
+                    try:
+                        list_url = info['webpage_url']
+                    except:
+                        print('获取列表地址失败!')
+                        return
+                    
+                if raw_url.__contains__('v='):
+                    info = extract_info(list_url)
+                # 获取entries
+                entries = info['entries']
+                if len(entries)==0:
+                    print('列表为空!')
                     return
-                if not args[2].startswith(('http://','https://')):
-                    print('封面url不合法!')
-                    return
-                download(url,args[1],args[2])
+                indexs = url.split('|')[1].split(',')
+                if len(indexs) == 0:
+                    # 下载全部
+                    for entry in entries:
+                        try:
+                            url = entry['url']
+                        except:
+                            try:
+                                url = entry['webpage_url']
+                            except:
+                                print('获取下载地址失败!')
+                                continue
+                        download(url)
+                        return
+                for index in indexs:
+                    try:
+                        index = int(index.strip())
+                        if index>len(entries) or index<=0:
+                            print('序号不合法!')
+                            continue
+                        entry = entries[index-1]
+                        # 下载
+                        try:
+                            url = entry['url']
+                        except:
+                            try:
+                                url = entry['webpage_url']
+                            except:
+                                print('获取下载地址失败!')
+                                continue
+                        download(url)
+                    except Exception as e:
+                        continue   
             else:
-                print('非法参数!')
+                # 不下载列表 去掉列表后缀
+                if  url.__contains__('youtube.com') and url.find('list=') != -1 and url.find('v=') != -1:
+                    url = url.split('&')[0]
+                if len(args) == 1:
+                    download(url,None,None)
+                elif len(args) == 2:
+                    title_or_url = args[1]
+                    if title_or_url.startswith(('http://','https://')):
+                        download(url,None,title_or_url)
+                    else:
+                        download(url,title_or_url,None)
+                elif (len(args) == 3):
+                    if args[1].startswith(('http://','https://')):
+                        print('歌曲名称不合法!')
+                        return
+                    if not args[2].startswith(('http://','https://')):
+                        print('封面url不合法!')
+                        return
+                    download(url,args[1],args[2])
+                else:
+                    print('非法参数!')
+        elif flag.endswith('.csv'):
+            # 批量下载
+            # 判断csv文件是否存在
+            if not os.path.exists(flag):
+                print('csv文件不存在!')
+                return
+            batch_download(flag)
         else:
             print('请输入合法的网址!')
             return
@@ -475,11 +618,15 @@ if  __name__ == '__main__':
     # download('https://www.bilibili.com/video/BV1yR4y1L7KN/?spm_id_from=333.1007.top_right_bar_window_default_collection.content.click')
     # clip('青花瓷-周杰伦.mp3','00:00:00','00:00:30')
     # loop = asyncio.get_event_loop()
-    # a=  loop.run_until_complete(search("https://music.163.com/song?id=186004&userid=112911170"))
+    # a=  loop.run_until_complete(search("グーラ領⧸森林"))
     # 转换为秒
     # 调用异步函数search_bilibili_
     # res = asyncio.run(search_bilibili("a lover's Concerto"))
     # 获取执行的结果
     # sync_meta()
-    # download('https://www.youtube.com/watch?v=lAshc3ubJIw','グーラ領/森林')
+    # download('https://www.youtube.com/watch?v=6MFSHrX5swY')
+    # clip('グーラ領⧸森林.mp3','00:00:00','00:00:30')
+    # batch_download('test.csv')
+    # info = extract_info('https://www.youtube.com/playlist?list=PL68LFSU9iLnC3YSNDqfy3x-1uF8czx33c')
+    # print(info)
     pass
\ No newline at end of file
